unit fOpenAITranslator_Controls;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, ImageList, ImgList,
  StdCtrls, Vcl.Menus;

type
  TFRMOpenAITranslator_Controls = class(TForm)
    btnControlAudio: TButton;
    ImageList1: TImageList;
    PopupMenu1: TPopupMenu;
    Start1: TMenuItem;
    Stop1: TMenuItem;
    N1: TMenuItem;
    Configuration1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    lblTranslation: TLabel;
    procedure btnConfigurationClick(Sender: TObject);
    procedure btnControlAudioClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure Configuration1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Start1Click(Sender: TObject);
    procedure Stop1Click(Sender: TObject);
  private
    FOnControlConfiguration: TNotifyEvent;
    FOnControlStart: TNotifyEvent;
    FOnControlStop: TNotifyEvent;
    FOnControlExit: TNotifyEvent;
  private
    FListening: Boolean;
    procedure SetListening(const Value: Boolean);
    procedure SetTranslation(const Value: string);
  protected
    procedure DoControlStart; virtual;
    procedure DoControlStop; virtual;
    procedure DoControlConfiguration; virtual;
    procedure DoControlExit; virtual;
  public
    property Listening: Boolean write SetListening;
    property Translation: string write SetTranslation;
  public
    property OnControlConfiguration: TNotifyEvent read FOnControlConfiguration
        write FOnControlConfiguration;
    property OnControlExit: TNotifyEvent read FOnControlExit write FOnControlExit;
    property OnControlStart: TNotifyEvent read FOnControlStart write
        FOnControlStart;
    property OnControlStop: TNotifyEvent read FOnControlStop write FOnControlStop;
  end;

var
  FRMOpenAITranslator_Controls: TFRMOpenAITranslator_Controls;

implementation

{$R *.dfm}

procedure TFRMOpenAITranslator_Controls.btnConfigurationClick(Sender: TObject);
begin
  DoControlConfiguration;
end;

procedure TFRMOpenAITranslator_Controls.btnControlAudioClick(Sender: TObject);
begin
  if FListening then
    DoControlStop
  else
    DoControlStart;
end;

procedure TFRMOpenAITranslator_Controls.btnStopClick(Sender: TObject);
begin
  DoControlStop;
end;

procedure TFRMOpenAITranslator_Controls.Configuration1Click(Sender: TObject);
begin
  DoControlConfiguration;
end;

procedure TFRMOpenAITranslator_Controls.DoControlConfiguration;
begin
  if Assigned(FOnControlConfiguration) then
    FOnControlConfiguration(self);
end;

procedure TFRMOpenAITranslator_Controls.DoControlExit;
begin
  if Assigned(FOnControlExit) then
    FOnControlExit(self);
end;

procedure TFRMOpenAITranslator_Controls.DoControlStart;
begin
  if Assigned(FOnControlStart) then
    FOnControlStart(self);
end;

procedure TFRMOpenAITranslator_Controls.DoControlStop;
begin
  if Assigned(FOnControlStop) then
    FOnControlStop(self);
end;

procedure TFRMOpenAITranslator_Controls.Exit1Click(Sender: TObject);
begin
  DoControlExit;
end;

procedure TFRMOpenAITranslator_Controls.FormShow(Sender: TObject);
begin
  Left := 0;
  Top := Screen.WorkAreaRect.Bottom - Height;
  Width := Screen.DesktopWidth;
end;

procedure TFRMOpenAITranslator_Controls.SetListening(const Value: Boolean);
begin
  FListening := Value;
  if FListening then
    btnControlAudio.ImageIndex := 1
  else
    btnControlAudio.ImageIndex := 0;
  Application.ProcessMessages;
end;

procedure TFRMOpenAITranslator_Controls.SetTranslation(const Value: string);
begin
  lblTranslation.Caption := Value;
  Application.ProcessMessages;
end;

procedure TFRMOpenAITranslator_Controls.Start1Click(Sender: TObject);
begin
  DoControlStart;
end;

procedure TFRMOpenAITranslator_Controls.Stop1Click(Sender: TObject);
begin
  DoControlStop;
end;

end.
